function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import useEventCallback from '@restart/hooks/useEventCallback';
import useMounted from '@restart/hooks/useMounted';
import React, { useContext, useEffect, useMemo, useState, isValidElement, createElement, useCallback } from 'react';
import { isValidElementType } from 'react-is'; // import { decode } from 'sourcemap-codec';

import { transform } from './transform'; // try and match render() calls with arguments to avoid false positives with class components

const hasRenderCall = code => !!code.match(/render\((?!\s*\))/gm);

const prettierComment = /(\{\s*\/\*\s+prettier-ignore\s+\*\/\s*\})|(\/\/\s+prettier-ignore)/gim;
const hooks = {};
Object.entries(React).forEach(([key, value]) => {
  if (key.startsWith('use')) hooks[key] = value;
});
export const isTypeScriptEnabled = language => {
  if (!language) return false;
  const lower = language.toLowerCase();
  return lower === 'typescript' || lower === 'tsx' || lower === 'ts';
};
export const Context = /*#__PURE__*/React.createContext({});

const getRequire = imports => function require(request) {
  if (!imports) throw new Error('no imports');
  if (!(request in imports)) throw new Error(`Module not found: ${request}`);
  const obj = imports[request];
  return obj && (obj.__esModule || obj[Symbol.toStringTag] === 'Module') ? obj : {
    default: obj
  };
};

function handleError(err, fn) {
  const fnStr = fn.toString(); // account for the function chrome lines

  const offset = fnStr.slice(0, fnStr.indexOf('{')).split(/\n/).length;
  let pos;

  if ('line' in err) {
    pos = {
      line: err.line,
      column: err.column
    };
  } else if ('lineNumber' in err) {
    pos = {
      line: err.lineNumber - 1,
      column: err.columnNumber - 1
    };
  } else {
    var _err$stack;

    const [, line, col] = (_err$stack = err.stack) === null || _err$stack === void 0 ? void 0 : _err$stack.match(/at eval.+<anonymous>:(\d+):(\d+)/m);
    pos = {
      line: +line - 1,
      column: +col - 1
    };
  }

  if (!pos) return err; // if (result.map) {
  //   const decoded = decode(result.map.mappings);
  //   const line = pos.line - offset;
  //   const mapping = decoded[line]?.find(([col]) => col === pos.column);
  //   if (mapping) {
  //     err.location = { line: mapping[2], column: mapping[3] };
  //   }
  // }

  return err;
}

function codeToComponent(compiledCode, {
  scope,
  preample,
  exportToRender,
  renderAsComponent = false
}) {
  return new Promise((resolve, reject) => {
    const isInline = !hasRenderCall(compiledCode);

    if (renderAsComponent && !isInline) {
      throw new Error('Code using `render()` cannot use top level hooks. ' + 'Either provide your own stateful component, or return a jsx element directly.');
    }

    const render = element => {
      if (element === undefined) {
        reject(new SyntaxError('`render()` was called without a JSX element'));
        return;
      }

      resolve(element);
    }; // const [clearTimes, timers] = createTimers();
    // DU NA NA NAAAH


    const finalScope = _objectSpread(_objectSpread({}, hooks), scope);

    const exports = {};
    const args = ['React', 'render', 'exports'].concat(Object.keys(finalScope));
    const values = [React, render, exports].concat(Object.values(finalScope));
    let body = compiledCode;

    if (renderAsComponent) {
      body = `return React.createElement(function StateContainer() {\n${body}\n})`;
    }

    if (preample) body = `${preample}\n\n${body}`; // eslint-disable-next-line no-new-func

    const fn = new Function(...args, body);
    let element;

    try {
      element = fn(...values);
    } catch (err) {
      reject(handleError(err, fn));
      return;
    }

    const exportedValues = Object.values(exports);

    if ('default' in exports) {
      var _exports$default;

      element = (_exports$default = exports.default) !== null && _exports$default !== void 0 ? _exports$default : element;
    } else if (exportedValues.length) {
      var _ref, _exports;

      element = (_ref = (_exports = exports[exportToRender]) !== null && _exports !== void 0 ? _exports : exportedValues[0]) !== null && _ref !== void 0 ? _ref : element;
    }

    if (element === undefined) {
      if (isInline) {
        reject(new SyntaxError('The code did not return a JSX element'));
      }

      return;
    }

    if (! /*#__PURE__*/isValidElement(element)) {
      if (isValidElementType(element)) {
        element = /*#__PURE__*/createElement(element);
      } else if (isInline) {
        reject(new SyntaxError('The code did not return a valid React element or element type'));
      }
    }

    resolve(element);
  });
}

export function useLiveContext() {
  return useContext(Context);
}
export function useElement() {
  return useLiveContext().element;
}
export function useError() {
  return useLiveContext().error;
}
export const objectZip = (arr, arr2) => Object.fromEntries(arr.map((v, i) => [v, arr2[i]]));

function defaultResolveImports(sources) {
  // @ts-ignore
  return Promise.all(sources.map((s) => import(/* webpackIgnore: true */ /* @vite-ignore */ s)));
}

function useCompiledCode(consumerCode, showImports, isTypeScript, setError) {
  const compile = useCallback(nextCode => {
    const isInline = !hasRenderCall(nextCode);
    nextCode = nextCode.replace(prettierComment, '').trim();
    return transform(nextCode, {
      compiledFilename: 'compiled.js',
      filename: 'example.js',
      wrapLastExpression: isInline,
      syntax: isTypeScript ? 'typescript' : 'js',
      transforms: isTypeScript ? ['typescript', 'imports', 'jsx'] : ['imports', 'jsx']
    });
  }, [isTypeScript]);
  const initialResult = useMemo(() => {
    const nextCode = consumerCode.replace(prettierComment, '').trim();
    const emptyResponse = {
      code: nextCode,
      imports: []
    };

    if (showImports) {
      return {
        code: nextCode,
        imports: []
      };
    }

    try {
      return transform(nextCode, {
        syntax: isTypeScript ? 'typescript' : 'js',
        compiledFilename: 'compiled.js',
        filename: 'example.js',
        removeImports: true,
        transforms: []
      });
    } catch (error) {
      setError(error);
      return {
        code: nextCode,
        imports: []
      };
    }
  }, [consumerCode, compile, setError, showImports]);
  return [{
    compiledCode: showImports ? initialResult.code : initialResult.code.trimStart(),
    removedImports: showImports ? [] : initialResult.imports
  }, compile];
}
/**
 * The Provider supplies the context to the other components as well as handling
 * jsx transpilation and import resolution.
 */


export default function Provider({
  scope,
  children,
  code: rawCode,
  language,
  theme,
  exportToRender,
  showImports = true,
  renderAsComponent = false,
  resolveImports = defaultResolveImports
}) {
  const isMounted = useMounted();
  const [error, setError] = useState(null);
  const [{
    element
  }, setState] = useState({
    element: null
  });
  const isTypeScript = isTypeScriptEnabled(language);
  const [initialResult, compile] = useCompiledCode(rawCode, showImports, isTypeScript, setError);
  const initialCompiledCode = initialResult.compiledCode;
  const handleChange = useEventCallback(nextCode => {
    try {
      const {
        code: compiledCode,
        imports
      } = compile(nextCode);
      const sources = Array.from(new Set([...initialResult.removedImports, ...imports].map(i => i.source)));
      Promise.resolve(resolveImports(sources)).then(results => Array.isArray(results) ? objectZip(sources, results) : results).then(fetchedImports => codeToComponent(compiledCode, {
        renderAsComponent,
        isTypeScript,
        exportToRender,
        // also include the orginal imports if they were removed
        preample: initialResult.removedImports.map(i => i.code).join('\n').trimStart(),
        scope: _objectSpread(_objectSpread({}, scope), {}, {
          require: getRequire(fetchedImports)
        })
      })).then(element => {
        if (!isMounted()) return;
        setState({
          element
        });
        setError(null);
      }, err => {
        if (!isMounted()) return;
        setError(err);
      });
    } catch (err) {
      setError(err);
    }
  });
  useEffect(() => {
    handleChange(initialCompiledCode);
  }, [initialCompiledCode, scope, handleChange]);
  const context = useMemo(() => ({
    theme,
    error,
    element,
    language,
    code: initialCompiledCode,
    onError: setError,
    onChange: handleChange
  }), [initialCompiledCode, element, error, handleChange, language, theme]);
  return /*#__PURE__*/React.createElement(Context.Provider, {
    value: context
  }, children);
}