function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { tt } from './parser';
export default class ImportRemoverTransformer {
  constructor(context, {
    removeImports
  }) {
    _defineProperty(this, "tokens", void 0);

    _defineProperty(this, "importProcessor", void 0);

    _defineProperty(this, "imports", []);

    _defineProperty(this, "removeImports", void 0);

    _defineProperty(this, "num", 0);

    this.tokens = context.tokenProcessor;
    this.importProcessor = context.importProcessor;
    this.removeImports = removeImports || false; // clear the replacements b/c we are handling imports
    // @ts-ignore private

    this.importProcessor.identifierReplacements.clear();
  }

  getPrefixCode() {
    return '';
  }

  getHoistedCode() {
    return '';
  }

  getSuffixCode() {
    return '';
  }

  process() {
    if (!this.tokens.matches1(tt._import)) return false; // dynamic import

    if (this.tokens.matches2(tt._import, tt.parenL)) {
      return true;
    }

    this.tokens.removeInitialToken();

    while (!this.tokens.matches1(tt.string)) {
      this.tokens.removeToken();
    }

    const path = this.tokens.stringValue();
    const detail = this.buildImport(path);
    this.importProcessor.claimImportCode(path);

    if (detail !== null && detail !== void 0 && detail.code) {
      this.imports.push(detail);
      this.tokens.replaceTokenTrimmingLeftWhitespace(this.removeImports ? '' : detail.code);
    } else {
      this.tokens.removeToken();
    }

    if (this.tokens.matches1(tt.semi)) {
      this.tokens.removeToken();
    }

    return true;
  }

  getIdentifier(src) {
    return `${src.split('/').pop().replace(/\W/g, '_')}$${this.num++}`;
  }

  buildImport(path) {
    // @ts-ignore
    if (!this.importProcessor.importInfoByPath.has(path)) {
      return null;
    }

    let FN = 'require';
    const {
      defaultNames,
      wildcardNames,
      namedImports,
      hasBareImport
    } = // @ts-ignore
    this.importProcessor.importInfoByPath.get(path);
    const req = `${FN}('${path}');`;
    const tmp = this.getIdentifier(path);
    const named = [];
    const details = {
      base: null,
      source: path,
      keys: [],
      code: ''
    };
    namedImports.forEach(s => {
      if (this.importProcessor.isTypeName(s.localName)) {
        return;
      }

      named.push(s.localName === s.importedName ? s.localName : `${s.importedName}: ${s.localName}`);
      details.keys.push({
        local: s.localName,
        imported: s.importedName
      });
    });

    if (defaultNames.length || wildcardNames.length) {
      const name = defaultNames[0] || wildcardNames[0];

      if (!this.importProcessor.isTypeName(name)) {
        details.base = name; // intentionally use `var` so that conflcits with Jarle provider scope get resolved naturally
        // with the import overriding the scoped identifier

        if (wildcardNames.length) {
          details.code = `var ${name} = ${req}`;
        } else {
          details.code = `var ${tmp} = ${req} var ${name} = ${tmp}.default;`;
        }
      }
    }

    if (named.length) {
      details.code += ` var { ${named.join(', ')} } = ${details.code ? `${tmp};` : req}`;
    }

    if (hasBareImport) {
      details.code = req;
    }

    details.code = details.code.trim();
    return details;
  }

}