"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = wrapLastExpression;

var _parser = require("./parser");

function findLastExpression(tokens) {
  let lastExprIdx = null;

  for (let i = 0; i < tokens.tokens.length; i++) {
    if (tokens.matches2AtIndex(i, _parser.tt._export, _parser.tt._default)) {
      return null;
    } // @ts-ignore


    if (tokens.tokens[i].isTopLevel) {
      const code = tokens.code.slice(tokens.tokens[i].start, tokens.tokens[i].end);

      if (code.startsWith('exports')) {
        return null;
      }

      if (tokens.matches1AtIndex(i, _parser.tt._return)) {
        return null;
      }

      lastExprIdx = i;
    }
  }

  return lastExprIdx;
}

function process(tokens, lastIndex) {
  if (tokens.currentIndex() !== lastIndex) {
    return false;
  }

  let prev = tokens.currentIndex() - 1;
  let lastWasSemi = prev >= 0 && !tokens.matches1AtIndex(prev, _parser.tt.semi);

  if (tokens.matches2(_parser.tt._export, _parser.tt._default)) {
    tokens.removeInitialToken();
    tokens.replaceTokenTrimmingLeftWhitespace(lastWasSemi ? 'return' : '; return');
  } else {
    let code = `return ${tokens.currentTokenCode()}`;

    if (lastWasSemi) {
      code = `;${code}`;
    }

    tokens.replaceTokenTrimmingLeftWhitespace(code);
  }

  return true;
}

function wrapLastExpression({
  tokenProcessor
}) {
  let lastExprIdx = findLastExpression(tokenProcessor);

  if (lastExprIdx == null) {
    return tokenProcessor.code;
  }

  while (!tokenProcessor.isAtEnd()) {
    let wasProcessed = process(tokenProcessor, lastExprIdx);

    if (!wasProcessed) {
      tokenProcessor.copyToken();
    }
  }

  return tokenProcessor.finish();
}